@@ -104,6 +104,10 @@ def freeze(dist):
     eskycode += marshal.dumps(compile("","esky/__init__.py","exec"))
     eskybscode = imp.get_magic() + struct.pack("<i",0)
     eskybscode += marshal.dumps(compile("","esky/bootstrap.py","exec"))
+    #  Copy any core dependencies
+    for nm in os.listdir(dist.freeze_dir):
+        if is_core_dependency(nm):
+            dist.copy_to_bootstrap_env(nm)
     #  Copy the loader program for each script into the bootstrap env, and
     #  append the bootstrapping code to it as a zipfile.
     for exe in dist.get_executables():
@@ -114,10 +118,6 @@ def freeze(dist):
         bslib.writestr(zipfile.ZipInfo("esky/__init__.pyc",cdate),eskycode)
         bslib.writestr(zipfile.ZipInfo("esky/bootstrap.pyc",cdate),eskybscode)
         bslib.close()
-    #  Copy any core dependencies
-    for nm in os.listdir(dist.freeze_dir):
-        if is_core_dependency(nm):
-            dist.copy_to_bootstrap_env(nm)
 
 
 def _normalise_opt_name(nm):
@@ -137,8 +137,8 @@ def _normalise_opt_name(nm):
 #  python version as the target exe, we can munge sys.path to bootstrap it
 #  into the existing process.
 _CUSTOM_WIN32_CHAINLOADER = """
-_orig_chainload = chainload
-def chainload(target_dir):
+_orig_chainload = _chainload
+def _chainload(target_dir):
   mydir = dirname(sys.executable)
   pydll = "python%%s%%s.dll" %% sys.version_info[:2]
   if not exists(pathjoin(target_dir,pydll)):
