@@ -22,7 +22,7 @@ use during the bootstrap process:
   Chainloading:         execv, chainload
   Filesystem:           listdir, exists, basename, dirname, pathjoin
   Version handling:     split_app_version, join_app_version, parse_version,
-                        get_all_version, get_best_version
+                        get_all_versions, get_best_version
 
 
 """
@@ -33,16 +33,13 @@ import errno
 #  The os module is not builtin, so we grab what we can from the
 #  platform-specific modules and fudge the rest.
 if "posix" in sys.builtin_module_names:
+    import fcntl
     from posix import listdir, stat, unlink, rename, execv
     SEP = "/"
-    try:
-        import fcntl
-    except ImportError:
-        fcntl = None
 elif "nt" in sys.builtin_module_names:
+    fcntl = None
     from nt import listdir, stat, unlink, rename, spawnv, P_WAIT
     SEP = "\\"
-    fcntl = None
     def execv(filename,args):
         res = spawnv(P_WAIT,filename,args)
         raise SystemExit(res)
@@ -118,17 +115,22 @@ def chainload(target_dir):
     it will not be removed by any simultaneously-running instances of the
     application.
     """
+    #  This global will hold the locked file to keep it open during execution.
     global _version_dir_lockfile
     lockfile = pathjoin(target_dir,"esky-bootstrap.txt")
     try:
         #  On windows, holding the file open is enough to lock it.
-        #  On other platforms, try for a shared lock using fcntl.
-        #  We put the fileobj in a global to hold it open.
+        #  On other platforms, try for a shared lock using fcntl.flock.
+        #  While fcntl.fcntl locks are apparently the new hotness, they have
+        #  unfortunate semantics that we don't want for this application:
+        #      * LOCK_EX requires write access to the file
+        #      * not inherited across fork()
+        #      * released when closing *any* fd associated with that file
+        #  fcntl.flock doesn't have these problems, but may fail on NFS.
+        #  I'll wait for the bug reports to come in on that one...
         _version_dir_lockfile = open(lockfile,"r")
-        if "nt" not in sys.builtin_module_names:
-            if fcntl is not None:
-                fd = _version_dir_lockfile.fileno()
-                fcntl.lockf(fd,fcntl.LOCK_SH)
+        if fcntl is not None:
+            fcntl.flock(_version_dir_lockfile,fcntl.LOCK_SH)
     except EnvironmentError:
         #  If the lockfile has gone missing, the version is being uninstalled.
         #  Our only option is to re-execute ourself and find the new version.
@@ -161,10 +163,13 @@ def get_best_version(appdir,include_partial_installs=False):
     #  Find all potential version directories, sorted by version number.
     candidates = []
     for nm in listdir(appdir):
-        (_,ver,_) = split_app_version(nm)
-        if ver and is_version_dir(pathjoin(appdir,nm)):
-            ver = parse_version(ver)
-            candidates.append((ver,nm))
+        (_,ver,platform) = split_app_version(nm)
+        #  If its name didn't parse properly, don't bother looking inside.
+        if ver and platform:
+            #  We have to pay another stat() call to check if it's active.
+            if is_version_dir(pathjoin(appdir,nm)):
+                ver = parse_version(ver)
+                candidates.append((ver,nm))
     candidates = [c[1] for c in sorted(candidates,reverse=True)]
     #  In the (hopefully) common case of no failed updates, we don't need
     #  to poke around in the filesystem so we just return asap.
@@ -192,10 +197,11 @@ def get_all_versions(appdir,include_partial_installs=False):
     #  Find all potential version directories, sorted by version number.
     candidates = []
     for nm in listdir(appdir):
-        (_,ver,_) = split_app_version(nm)
-        if ver and is_version_dir(pathjoin(appdir,nm)):
-            ver = parse_version(ver)
-            candidates.append((ver,nm))
+        (_,ver,platform) = split_app_version(nm)
+        if ver and platform:
+            if is_version_dir(pathjoin(appdir,nm)):
+                ver = parse_version(ver)
+                candidates.append((ver,nm))
     candidates = [c[1] for c in sorted(candidates,reverse=True)]
     #  Filter out any that are not completely installed.
     if not include_partial_installs:
@@ -225,6 +231,15 @@ def is_installed_version_dir(vdir):
     return not exists(pathjoin(vdir,"esky-bootstrap"))
 
 
+def is_uninstalled_version_dir(vdir):
+    """Check whether the given version directory is partially uninstalled.
+
+    A partially-uninstalled version dir has had its "esky-bootstrap.txt"
+    file renamed to "esky-bootstrap-old.txt".
+    """
+    return exists(pathjoin(vdir,"esky-bootstrap-old.txt"))
+
+
 def split_app_version(s):
     """Split an app version string to name, version and platform components.
 
